#include "Enemy.h"
#include <cmath>
#include <iostream>

using namespace Tmpl8;				// to use template classes
using namespace glm;				// to use glm vector stuff

Enemy::Enemy(Surface* a_Surface, Surface* screen, EnemyGroupData* egd, Enemy* owner, Player* player, EnemyType type) : Sprite(a_Surface, 1)
{
	this->screen = screen;
	this->player = player;
	this->type = type;

	enemyGroupData = egd;

	this->position = enemyGroupData->GetSpawnPosition();

	arcTimer = Rand(1);

	prevous = owner;
	if (enemyGroupData->Next())
		next = new Enemy(enemyGroupData->NextSurface(), screen, egd, this, player, enemyGroupData->nextType);
	returnTimer = returnInterval;
}

Enemy::~Enemy()
{
}

void Enemy::DrawEnemy()
{
	glm::vec2 p = GetPosition();
	if (state != DEAD)
		this->Draw(p.x, p.y, screen);
	if (bullet != 0)
		bullet->Draw();
	if (next != 0)
		next->DrawEnemy();

	if (explosionParticles != 0)
		explosionParticles->Draw();
}

void Enemy::OwnerUpdate(float DT)
{
	/*if (enemyGroupData->changeDirection)
	{
		enemyGroupData->changeDirection = false;
		enemyGroupData->direction = enemyGroupData->direction == glm::vec2(1, 0) ? glm::vec2(-1, 0) : glm::vec2(1, 0);
	}*/
	if (enemyGroupData->changeDirectionTo != enemyGroupData->direction)
	{
		enemyGroupData->direction = enemyGroupData->changeDirectionTo;
	}
}

glm::vec2 Enemy::GetPosition()
{
	return state == IDLE ? position : arcPosition;
}

void Enemy::Die()
{

	explosionParticles = new Particles(screen);
	explosionParticles->maxAmount = 300;
	explosionParticles->minAmount = 280;
	explosionParticles->minAngle = 0;
	explosionParticles->maxAngle = 360;
	explosionParticles->minSpeed = 0.3f;
	explosionParticles->maxSpeed = 1.f;
	explosionParticles->minLifetime = 0.2f;
	explosionParticles->maxLifeTime = 0.5f;
	explosionParticles->color = 0xffffff;
	explosionParticles->startPosition = GetPosition();
	explosionParticles->Emit();

	switch (type)
	{
	case COMMANDER:
		player->score += 25;
		break;
	case RED:
		player->score += 10;
		break;
	case PURPLE:
		player->score += 3;
		break;
	case GREEN:
		player->score += 1;
		break;
	}

	state = DEAD;
}

bool Enemy::IsDead()
{
	return state == DEAD ? true : false;
}

bool Enemy::AllDead()
{
	if (!IsDead())
		return false;
	if (next != 0)
		return next->AllDead();
	else
		return true;
}

void Enemy::CollisionUpdate()
{
	if (player->Killed())
		return;
	float lengthX = (player->position.x + player->GetWidth() / 2) - (arcPosition.x + GetWidth() / 2);
	float lengthY = (player->position.y + player->GetHeight() / 2) - (arcPosition.y + GetHeight() / 2);
	float playerdistance = glm::sqrt((lengthX * lengthX) + lengthY * lengthY);
	if (playerdistance < GetWidth() / 2 + player->GetWidth() / 2) 
	{
		player->Kill();
		state = DEAD;
	}

	if (bullet != 0)
	{
		float lengthX = (player->position.x + player->GetWidth() / 2) - (bullet->position.x);
		float lengthY = (player->position.y + player->GetHeight() / 2) - (bullet->position.y);
		float playerdistance = glm::sqrt((lengthX * lengthX) + lengthY * lengthY);
		if (playerdistance < GetWidth() / 2 + player->GetWidth() / 2)
		{
			player->Kill();
			delete bullet;
			bullet = 0;
		}
	}
}

bool Enemy::CanArc(float DT)
{
	if (player->Killed())
		return false;
	if (type != COMMANDER && next != 0)
	{
		if (prevous == 0)
			return false;
		if (!(abs(prevous->position.y - position.y) >= enemyGroupData->SPACE * 0.9f || abs(next->position.y - position.y) >= enemyGroupData->SPACE * 0.9f))
		{
			return false;
		}
	}
	arcTimer -= DT / 1000;
	if (arcTimer <= 0)
	{
		arcTimer = 2;
		if (Rand(20) > 15)
		{
			return true;
		}
	}
	return false;
}

void Enemy::Update(float DT)
{
	if (prevous == 0)
		OwnerUpdate(DT);

	position += enemyGroupData->direction;

	if (bullet != 0)
	{
		bullet->Update();
		if (bullet->bulletGone)
		{
			delete bullet;
			bullet = 0;
		}
	}

	//std::cout << angle << std::endl;
	switch (state)
	{
	case IDLE:
		// have I reached the end? 
		if (position.x + this->GetWidth() > screen->GetWidth() || position.x <= 0)
		{
			//enemyGroupData->changeDirection = true;
			enemyGroupData->changeDirectionTo = position.x < screen->GetWidth() / 2 ? glm::vec2(1, 0) : glm::vec2(-1, 0);
		}

		if (CanArc(DT))
		{
			this->state = ARC;
			if (this->position.x + this->GetWidth() / 2 <= this->screen->GetWidth() / 2)
			{
				angle = 360;
				arcDirection = -1;
			}
			else
			{
				angle = 180;
				arcDirection = 1;
			}
		}
		break;
	case ARC:
		if (arcPivot == glm::vec2(0, 0))
			arcPivot = position + glm::vec2(-radius, 0);
		angle+=arcDirection*speed;
		arcPosition.x = arcPivot.x + cos(angle * (PI / 180)) * radius;
		arcPosition.y = arcPivot.y + sin(angle * (PI / 180)) * radius;

		if (angle < 180 || angle > 360)
		{
			state = ARCEND;
			glm::vec2 moveTowards = glm::vec2(player->position.x + -arcDirection * 200, player->position.y) - position;
			direction = NormalizeVector(moveTowards);
			bulletTimer = bulletInterval;
			arcSpeed = speed;
			updateArcTimer = updateArcInterval;
			arcPivot = glm::vec2(0, 0);
		}
		break;
	case ARCEND:
		if (arcSpeed > speed / 3)
		arcSpeed -= DT / 1000 * 3;
		updateArcTimer -= DT / 1000;
		
		moveTowards = glm::vec2(player->position.x + -arcDirection * 200, player->position.y) - position;
		moveTowards = NormalizeVector(moveTowards);
		if ((direction.x <= 0 && moveTowards.x > 0) || (direction.x >= 0 && moveTowards.x < 0))
			direction.x += moveTowards.x * (DT / 1000 * 3 * speed);

		updateArcTimer = updateArcInterval;
		arcPosition += direction * arcSpeed;
		if (bullet == 0)
		{
			bulletTimer -= DT / 1000;
			if (bulletTimer <= 0)
			{
				Shoot();
				bulletTimer = bulletInterval;
			}
		}
		if (arcPosition.y > screen->GetHeight() || arcPosition.x > screen->GetWidth())
		{
			returnTimer -= DT / 1000;
			if (returnTimer <= 0)
			{
				returnTimer = returnInterval;
				arcPosition.x = position.x;
				arcPosition.y = -GetHeight();
				state = RETURN;
			}
		}
		else
			CollisionUpdate();
		break;
	case RETURN:
		arcPosition.y += speed;
		arcPosition.x = position.x;
		if (arcPosition.y >= position.y)
			state = IDLE;
	case DEAD:
		if (explosionParticles != 0)
		{
			explosionParticles->Update(DT);
			if (explosionParticles->finished)
			{
				delete explosionParticles;
				explosionParticles = 0;
				if (next != 0)
					next->prevous = prevous;
				if (prevous != 0)
					prevous->next = next;
				// TODO: Figure out why deleting this enemy causes error...
				//delete this;

			}
		}
	}

	if (next != 0)
		next->Update(DT);
}

void Enemy::Shoot()
{
	if (bullet == 0)
	{
		bullet = new Bullet(screen, 4);
		bullet->position = arcPosition;
		bullet->position.x += GetWidth() / 2;
		bullet->direction = glm::vec2(0, 1);
	}
}

glm::vec2 Enemy::NormalizeVector(glm::vec2 vector)
{
	float length = glm::sqrt(vector.x * vector.x + vector.y * vector.y);
	if (length != 0){
		vector.x = vector.x / length;
		vector.y = vector.y / length;
	}

	return vector;
}