#include "Player.h"
#include "game.h"
#include <iostream>

using namespace Tmpl8;				// to use template classes
using namespace glm;				// to use glm vector stuff

/*Player::Player(char* a_File, Surface* screen, Game* game) : Surface(a_File)
{
	this->screen = screen;
	this->game = game;
	position.y = screen->GetHeight() - GetHeight();
	position.x = screen->GetWidth() / 2 - GetWidth() / 2;
}*/

Player::Player(Surface* a_Surface, Surface* screen, Game* game) : Sprite(a_Surface, 1)
{
	this->screen = screen;
	this->game = game;
	position.y = screen->GetHeight() - GetHeight();
	position.x = screen->GetWidth() / 2 - GetWidth() / 2;
}

Player::~Player()
{
	if (bullet != 0)
		delete bullet;
}

void Player::Kill()
{
	isDead = true;
	lives--;

	explosionParticles = new Particles(screen);
	explosionParticles->maxAmount = 300;
	explosionParticles->minAmount = 280;
	explosionParticles->minAngle = 0;
	explosionParticles->maxAngle = 360;
	explosionParticles->minSpeed = 0.3f;
	explosionParticles->maxSpeed = 1.f;
	explosionParticles->minLifetime = 0.2f;
	explosionParticles->maxLifeTime = 0.5f;
	explosionParticles->color = 0xffffff;
	explosionParticles->startPosition = position;
	explosionParticles->Emit();
}

int Player::GetLives()
{
	return lives;
}

bool Player::Killed()
{
	return isDead;
}

void Player::DrawPlayer()
{
	if (!isDead)
		this->Draw(position.x, position.y, screen);
	if (bullet != 0)
		bullet->Draw();
	if (explosionParticles != 0)
		explosionParticles->Draw();
}

void Player::Update(float DT)
{

	if (bullet != 0)
	{
		bullet->Update();
		if (bullet->bulletGone || game->enemyGroupData->BulletCollisionCheckUpdate(bullet->position, 1, game->enemy))
		{
			delete bullet;
			bullet = 0;
		}
	}
	if (explosionParticles != 0)
	{
		explosionParticles->Update(DT);
		if (explosionParticles->finished)
		{
			delete explosionParticles;
			explosionParticles = 0;
		}
	}

	if (position.x <= 0 && direction.x == -1)
		return;
	if (position.x >= screen->GetWidth() - GetWidth() && direction.x == 1)
		return;
	this->position += this->direction * speed;
}

void Player::InputDown(int key)
{
	if (key == 80)
		direction.x = -1;
	else if (key == 79)
		direction.x = 1;
	else if (key == 40)
	{
		if (isDead && lives > 0)
		{
			isDead = false;
			position.x = screen->GetWidth() / 2 - GetWidth() / 2;
		}
	}
}

void Player::InputUp(int key)
{
	if (key == 80 && direction.x == -1)
		direction.x = 0;
	else if (key == 79 && direction.x == 1)
		direction.x = 0;
	else if (key == 44)
		Shoot();
}

void Player::Shoot()
{
	if (isDead)
		return;
	if (bullet != 0)
	{
		if (bullet->bulletGone)
		{
			delete bullet;
			bullet = 0;
		}
		else
			return;

	}
	bullet = new Bullet(screen, 5);
	bullet->position = position;
	bullet->position.x += GetWidth() / 2;
	bullet->direction = glm::vec2(0, -1);
}