#include "game.h"
#include <iostream>



using namespace Tmpl8;				// to use template classes
using namespace glm;				// to use glm vector stuff

// -----------------------------------------------------------
// Initialize the game
// -----------------------------------------------------------

Sprite* background;

void Game::Init()
{
	star = new Star(screen, 0);
	background = new Sprite(new Surface("assets/background.png"), 1);
}

void Game::GameStart()
{
	if (gameStarted)
		return;
	// game initialization code goes here;
	// will be executed once when app starts
	enemyGroupData = new EnemyGroupData();
	//player = new Player("assets/ball.png", screen, this);
	player = new Player(new Surface("assets/ball.png"), screen, this);
	enemy = new Enemy(enemyGroupData->commanderEnemySurface, screen, enemyGroupData, 0, player, COMMANDER);
	gameStarted = true;
}

void Game::Restart()
{
	gameStarted = false;
	delete enemy;
	delete enemyGroupData;
	delete player;
	GameStart();
}

// -----------------------------------------------------------
// Main game tick function
// -----------------------------------------------------------
void Game::Tick( float _DT )
{
	// clear the canvas
	//screen->Clear( 0 );

	for (int x = 0; x < screen->GetWidth(); x+= background->GetWidth())
	{
		for (int y = 0; y < screen->GetHeight(); y+= background->GetHeight())
			background->Draw(x, y, screen);
	}
	star->Update(_DT);
	star->Draw();

	if (!gameStarted)
	{
		screen->Print("Press start", screen->GetWidth() / 2 - 25, screen->GetHeight() / 2, 0xffffff);
		return;
	}
	else if (player->GetLives() <= 0)
	{
		screen->Print("Game Over", screen->GetWidth() / 2 - 25, screen->GetHeight() / 2 + 20, 0xffffff);
		screen->Print("Press enter to restart", screen->GetWidth() / 2 - 60, screen->GetHeight() / 2, 0xffffff);
		return;
	}
	if (enemy->AllDead())
	{
		screen->Print("You won! ", screen->GetWidth() / 2 - 25, screen->GetHeight() / 2 + 20, 0xffffff);
		screen->Print("Press enter to restart", screen->GetWidth() / 2 - 60, screen->GetHeight() / 2, 0xffffff);
		return;
	}
	enemy->Update(_DT);
	enemy->DrawEnemy();

	player->Update(_DT);
	player->DrawPlayer();

	PrintUI();
}

void Game::PrintUI()
{
	char str[10];
	itoa(player->GetLives(), str, 10); // 10 - decimal; 
	screen->Print("Lives: ", 2, 2, 0xffffff);
	screen->Print(str, 50, 2, 0xffffff);
	screen->Print("Score: ", screen->GetWidth() / 2, 2, 0xffffff);
	itoa(player->score, str, 10);
	screen->Print(str, screen->GetWidth() / 2 + 50, 2, 0xffffff);
	
}

void Game::Shutdown()
{
	delete enemy;
	delete enemyGroupData;
	delete player;
	delete star;
}

void Game::KeyDown(int key)
{
	if (!gameStarted)
	{
		if (key == 40)
			GameStart();
		return;
	}
	if ((player->GetLives() <= 0 || enemy->AllDead()) && key == 40)
		Restart();
	player->InputDown(key);
}

void Game::KeyUp(int key)
{
	if (gameStarted)
		player->InputUp(key);
}